unit CustomOpenDialog;

interface

uses
  Windows, Messages, CommDlg, FWEventLog;

type
  TEventLogOpenDialog = class
  private
    FFileName, FInitialDir, FDisplayName: String;
    FEventSourceType: TFWEventSources;
  public
    function Execute(Handle: THandle): Boolean;
    property FileName: String read FFileName;
    property InitialDir: String read FInitialDir write FInitialDir;
    property EventSourceType: TFWEventSources read FEventSourceType;
    property DisplayName: String read FDisplayName;
  end;

var
  EventLogOpenDialog: TEventLogOpenDialog = nil;

implementation

uses SysUtils;

type
  PCustomResultStruct = ^TCustomResultStruct;
  TCustomResultStruct = record
    SourceType: DWORD;
    DisplayName: ShortString;
  end;

const
  MultiSelectBufferSize = High(Word) - 16;
  IDC_LOGTYPE_ID = 101;
  IDC_DISPLAY_NAME_ID = 102;
  HEAP_ZERO_MEMORY = 8;

var
  OpenFileName: TOpenFileName;
  OpenFilePath: array [0..MultiSelectBufferSize - 1] of Char;

{ TEventLogOpenDialog }

function HookOpenDialog(Wnd: HWND; Msg: UINT;
  WParam: WPARAM; LParam: LPARAM): UINT stdcall;
var
  ComboLogType: THandle;
  EditDisplayName: THandle;
  szNewStr: String;
  pResult: PCustomResultStruct;
begin
  Result := 0;
  case Msg of

    WM_INITDIALOG:
    begin
      ComboLogType := GetDlgItem(Wnd, IDC_LOGTYPE_ID);
      szNewStr := '( )';
      SendMessage(ComboLogType, CB_ADDSTRING, 0, DWORD(PChar(szNewStr)));
      szNewStr := '';
      SendMessage(ComboLogType, CB_ADDSTRING, 0, DWORD(PChar(szNewStr)));
      szNewStr := '';
      SendMessage(ComboLogType, CB_ADDSTRING, 0, DWORD(PChar(szNewStr)));
      szNewStr := '';
      SendMessage(ComboLogType, CB_ADDSTRING, 0, DWORD(PChar(szNewStr)));
      SendMessage(ComboLogType, CB_SETCURSEL, 0, 0);
    end;

    WM_NOTIFY:
    case POFNotify(lParam)^.hdr.code of
      CDN_FILEOK:
      begin
        ComboLogType := GetDlgItem(Wnd, IDC_LOGTYPE_ID);
        POFNotify(lParam)^.lpOFN^.dwReserved :=
          SendMessage(ComboLogType, CB_GETCURSEL, 0, 0);
        if POFNotify(lParam)^.lpOFN^.dwReserved = 0 then
        begin
          MessageBox(Wnd, '  ',
            ' ', MB_ICONERROR);
          Result := 1;
        end
        else
        begin
          pResult := HeapAlloc(GetProcessHeap, HEAP_ZERO_MEMORY,
            SizeOf(TCustomResultStruct));
          pResult^.SourceType := POFNotify(lParam)^.lpOFN^.dwReserved;
          EditDisplayName := GetDlgItem(Wnd, IDC_DISPLAY_NAME_ID);
          SetLength(pResult^.DisplayName,
            SendMessage(EditDisplayName, WM_GETTEXTLENGTH, 0, 0) + 1);
          SendMessage(EditDisplayName, WM_GETTEXT,
            Length(pResult^.DisplayName), DWORD(@pResult^.DisplayName[1]));
          POFNotify(lParam)^.lpOFN^.dwReserved := Integer(pResult);
        end;
        SetWindowLong(Wnd, DWL_MSGRESULT, Result);
      end;
    end;

    WM_COMMAND:
    begin
      case Word(WParam) of
        IDC_LOGTYPE_ID:
        begin
          ComboLogType := GetDlgItem(Wnd, IDC_LOGTYPE_ID);
          case SendMessage(ComboLogType, CB_GETCURSEL, 0, 0) of
            1:
            begin
              szNewStr := '  ""';
              EditDisplayName := GetDlgItem(Wnd, IDC_DISPLAY_NAME_ID);
              SendMessage(EditDisplayName, WM_SETTEXT, 0, DWORD(PChar(szNewStr)));
            end;
            2:
            begin
              szNewStr := '  ""';
              EditDisplayName := GetDlgItem(Wnd, IDC_DISPLAY_NAME_ID);
              SendMessage(EditDisplayName, WM_SETTEXT, 0, DWORD(PChar(szNewStr)));
            end;
            3:
            begin
              szNewStr := '  ""';
              EditDisplayName := GetDlgItem(Wnd, IDC_DISPLAY_NAME_ID);
              SendMessage(EditDisplayName, WM_SETTEXT, 0, DWORD(PChar(szNewStr)));
            end;
          end;
        end;
      end;
    end;
  end;
end;

function TEventLogOpenDialog.Execute(Handle: THandle): Boolean;

  function ExtractFileName(P: PChar; var S: TFilename): PChar;
  begin
    Result := AnsiStrScan(P, #0);
    if Result = nil then
    begin
      S := P;
      Result := StrEnd(P);
    end
    else
    begin
      SetString(S, P, Result - P);
      Inc(Result);
    end;
  end;

  procedure ExtractFileNames(P: PChar);
  var
    DirName, FileName: TFilename;
  begin
    P := ExtractFileName(P, DirName);
    P := ExtractFileName(P, FileName);
    if FileName = '' then
      FFileName := DirName
    else
    begin
      if AnsiLastChar(DirName)^ <> '\' then
        DirName := DirName + '\';
      repeat
        if (FileName[1] <> '\') and ((Length(FileName) <= 3) or
          (FileName[2] <> ':') or (FileName[3] <> '\')) then
          FileName := DirName + FileName;
        if FFileName = '' then
          FFileName := FileName
        else
          FFileName := FFileName + sLineBreak + FileName;
        P := ExtractFileName(P, FileName);
      until FileName = '';
    end;
  end;

begin
  ZeroMemory(@OpenFilePath[0], MultiSelectBufferSize);
  ZeroMemory(@OpenFileName, SizeOf(TOpenFileName));

  OpenFileName.lStructSize := SizeOf(TOpenFileName);
  OpenFileName.hwndOwner := Handle;
  OpenFileName.hInstance := hInstance;

  OpenFileName.lpstrFilter :=  '  (*.evt)'#0'*.evt'#0 +
    '  (*.*)'#0'*.*'#0#0;
  OpenFileName.lpstrTitle := '';
  OpenFileName.lpstrFile := OpenFilePath;
  OpenFileName.nMaxFile := MAX_PATH;
  OpenFileName.lpTemplateName := 'OPEN_FILE_TEMPLATE';
  OpenFileName.lpfnHook := HookOpenDialog;
  OpenFileName.Flags := OFN_EXPLORER or OFN_CREATEPROMPT or
    OFN_FILEMUSTEXIST or OFN_HIDEREADONLY or
    OFN_PATHMUSTEXIST or OFN_ENABLEHOOK or OFN_ENABLETEMPLATE or
    OFN_ENABLESIZING or OFN_ALLOWMULTISELECT;
  OpenFileName.lpstrInitialDir := PChar(FInitialDir);
  OpenFileName.lpstrDefExt := 'evt';
  Result := GetOpenFileName(OpenFileName);
  FFileName := '';
  FEventSourceType := esUnknown;
  FDisplayName := '';
  if Result then
  begin
    FEventSourceType := TFWLocalEventSources(
      PCustomResultStruct(OpenFileName.dwReserved)^.SourceType);
    FDisplayName := PCustomResultStruct(OpenFileName.dwReserved)^.DisplayName;
    HeapFree(GetProcessHeap, HEAP_ZERO_MEMORY,
      Pointer(OpenFileName.dwReserved));
    ExtractFileNames(OpenFileName.lpstrFile);
  end;
end;

end.
